local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

a.spells["auto_shot"] = {
	ID = 75--[[Auto Shot]],
	EnemyTargetNeeded = true,
	NotIfActive = true,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "Green", "Pink")
		return x.NoCC and not s.Shooting()
	end,
	NoGCD = true,
}

a.spells["berserking"] = {
	ID = 26297--[[Berserking]],
	Buff = 26297--[[Berserking]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		if x.Boss then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.InCombat and x.ActiveEnemy and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	NoGCD = true,
}

a.spells["blood_fury"] = {
	ID = 20572--[[Blood Fury]],
	Buff = 20572--[[Blood Fury]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		if x.Boss then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.InCombat and x.ActiveEnemy and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	NoGCD = true,
}

a.spells["Pet Attack"] = {
	Override = function(z)
		if x.NoCC and x.PetAlive and x.Enemy and not UnitGUID("pettarget") then
			s.FlashPet(PET_ACTION_ATTACK, "Green")
		elseif x.NoCC and x.PetAlive and x.Enemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK, "Pink")
		end
	end,
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and not a.PetAssistActive() then
			s.FlashPet(PET_MODE_ASSIST, "Pink")
		end
	end,
	NoGCD = true,
}

a.spells["hunters_mark"] = {
	ID = 1130--[[Hunter's Mark]],
	Debuff = 1130--[[Hunter's Mark]],
	EarlyRefresh = 5,
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aspect of the Hawk"] = {
	Type = "form",
	ID = 13165--[[Aspect of the Hawk]],
	Buff = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
	BuffUnit = "player",
	NotWhileMoving = true,
	NoGCD = true,
	FlashColor = "Green",
	Check = function(z)
		return x.ShouldTemporaryBuff and (not s.Moving("player") or s.InCombat())
	end,
	FlashID = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
}


a.spells["Aspect of the Cheetah while moving"] = {
	Type = "form",
	ID = 5118--[[Aspect of the Cheetah]],
	MyBuff = 5118--[[Aspect of the Cheetah]],
	BuffUnit = "player",
	NotWhileMoving = false,
	NoGCD = true,
	FlashColor = "Green",
	Check = function(z)
		return x.ShouldPermanentBuff and s.Moving("player") and not s.InCombat()
	end,
}

a.spells["explosive_trap,if=active_enemies>1"] = {
	ID = 13813--[[Explosive Trap]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	Run = function(z)
		if x.ActiveEnemies > 1 then
			z.Continue = false
			z.FlashColor = "White"
		else
			z.Continue = true
			z.FlashColor = "Blue"
		end
	end,
	FlashID = {
		13813--[[Explosive Trap]],
		82939--[[Explosive Trap]],
	},
	NoRangeCheck = true,
}

a.spells["focus_fire,five_stacks=1"] = {
	ID = 82692--[[Focus Fire]],
	ConsumeBuff = 19615--[[Frenzy]],
	ConsumeStack = 5,
	ConsumeStackTaken = 5,
	ConsumeBuffUnit = "pet",
	Check = function(z)
		return x.InCombat and x.PetAlive 
	end,
}

a.spells["focus_fire,five_stacks=1,if=!ticking&!buff.beast_within.up"] = {
	ID = 82692--[[Focus Fire]],
	Buff = 82692--[[Focus Fire]],
	BuffUnit = "player",
	ConsumeBuff = 19615--[[Frenzy]],
	ConsumeStack = 5,
	ConsumeStackTaken = 5,
	ConsumeBuffUnit = "pet",
	Check = function(z)
		return x.InCombat and x.PetAlive and s.BuffDuration(34471--[[The Beast Within]], "player") <= x.GCD
	end,
}

a.spells["serpent_sting,if=!ticking"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		return x.NoCC and ((x.RAP * 0.8 + 16200) * x.TimeToDie / 15) > (1.25 * x.avgDmg + 2306)
	end,
	EnemyTargetNeeded = true,
}

a.spells["serpent_sting,if=!ticking&target.time_to_die>=10"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		return x.NoCC and ((x.RAP * 0.8 + 16200) * 1.5 * 1.15 * x.TimeToDie / 15) > (1.25 * x.avgDmg + 2306)
	end,
	EnemyTargetNeeded = true,
}

a.spells["chimera_shot"] = {
	ID = 53209--[[Chimera Shot]],
	CheckLast = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["bestial_wrath,if=focus>60&!buff.beast_within.up"] = {
	ID = 19574--[[Bestial Wrath]],
	Buff = 34471--[[The Beast Within]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and x.InCombat and x.CurrentFocus > 60 
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["multi_shot,if=active_enemies>5|(active_enemies>2&buff.beast_cleave.down)"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if x.ActiveEnemies > 5 or (x.ActiveEnemies > 2 and s.BuffDuration(118455--[[Beast Cleave]], "pet") <= x.GCD) then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Blue"
			z.Continue = true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["multi_shot,if=buff.thrill_of_the_hunt.react&dot.serpent_sting.remains<2"] = {
	ID = 2643--[[Multi-Shot]],
	MyDebuff = 1978--[[Serpent Sting]],
	EarlyRefresh = 2,
	ConsumeBuff = 34720--[[Thrill of the Hunt]],
	ConsumeBuffUnit = "player",
	ConsumeStackTaken = 1,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["multi_shot,if=active_enemies>5"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if x.ActiveEnemies > 5 then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Blue"
			z.Continue = true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["multi_shot,if=active_enemies>3"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if x.ActiveEnemies > 3 then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Blue"
			z.Continue = true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["steady_shot,if=active_enemies>5"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and x.ActiveEnemies > 5
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["cobra_shot,if=active_enemies>5"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC and x.ActiveEnemies > 5
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["cobra_shot,if=active_enemies>3"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC and x.ActiveEnemies > 3
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["kill_shot"] = {
	ID = 53351--[[Kill Shot]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["a_murder_of_crows,if=enabled&!ticking"] = {
	ID = 131894--[[A Murder of Crows]],
	MyDebuff = 131894--[[A Murder of Crows]],
	Check = function(z)
		if x.Boss then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.NoCC and s.HasTalent(131894--[[A Murder of Crows]]) and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	EnemyTargetNeeded = true,
}

a.spells["fervor,if=enabled&focus<=50"] = {
	ID = 82726--[[Fervor]],
	Buff = 82726--[[Fervor]],
	BuffUnit = "player",
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(82726--[[Fervor]]) and x.CurrentFocus <= 50
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["fervor,if=enabled&!ticking&focus<=65"] = {
	ID = 82726--[[Fervor]],
	Buff = 82726--[[Fervor]],
	BuffUnit = "player",
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(82726--[[Fervor]]) and x.CurrentFocus <= 65
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["powershot,if=enabled"] = {
	ID = 109259--[[Powershot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and s.HasTalent(109259--[[Powershot]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["barrage,if=enabled"] = {
	ID = 120360--[[Barrage]],
	NotWhileMoving = false,
	NoStopChannel = true,
	Check = function(z)
		return x.NoCC and s.HasTalent(120360--[[Barrage]])
	end,
}

a.spells["lynx_rush,if=enabled&!dot.lynx_rush.ticking"] = {
	ID = 120697--[[Lynx Rush]],
	Debuff = 120699--[[Lynx Rush]],
	NotWhileMoving = false,
	Check = function(z)
		if x.Boss then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.PetActiveEnemy and x.PetAlive and s.HasTalent(120697--[[Lynx Rush]]) and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	NoRangeCheck = true,
	EnemyTargetNeeded = false,
}

a.spells["stampede,if=buff.rapid_fire.up|buff.bloodlust.react|target.time_to_die<=25"] = {
	ID = 121818--[[Stampede]],
	Check = function(z)
		if x.Boss then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.NoCC and x.ActiveEnemy and (a.Hasted() or (x.Boss and x.TimeToDie <= 25)) and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	EnemyTargetNeeded = true,
}

a.spells["glaive_toss,if=enabled"] = {
	ID = 117050--[[Glaive Toss]],
	Check = function(z)
		return x.NoCC and s.HasTalent(117050--[[Glaive Toss]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["dire_beast,if=enabled"] = {
	ID = 120679--[[Dire Beast]],
	Check = function(z)
		return x.NoCC and s.HasTalent(120679--[[Dire Beast]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["dire_beast,if=enabled&focus<=90"] = {
	ID = 120679--[[Dire Beast]],
	Check = function(z)
		return x.NoCC and x.Focus <= 90 and s.HasTalent(120679--[[Dire Beast]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["kill_command"] = {
	ID = 34026--[[Kill Command]],
	Check = function(z)
		return x.PetNoCC and x.PetAlive and x.PetActiveEnemy
	end,
	PetFrameNeeded = true,
	EnemyTargetNeeded = true,
	NoRangeCheck = true,
}

a.spells["arcane_shot,if=buff.thrill_of_the_hunt.react"] = {
	ID = 3044--[[Arcane Shot]],
	ConsumeBuff = 34720--[[Thrill of the Hunt]],
	ConsumeBuffUnit = "player",
	ConsumeStackTaken = 1,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["arcane_shot,if=focus>=60|(focus>=43&(cooldown.chimera_shot.remains>=action.steady_shot.cast_time))&(!buff.rapid_fire.up&!buff.bloodlust.react)"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and x.Focus >= 60 or (x.Focus >= 43 and (s.SpellCooldown(53209--[[Chimera Shot]]) >= s.CastTime(56641--[[Steady Shot]])) and not a.Hasted())
	end,
	EnemyTargetNeeded = true,
}

a.spells["arcane_shot,if=focus>=61|buff.beast_within.up"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and (x.Focus >= 61 or s.BuffDuration(34471--[[The Beast Within]], "player") > x.GCD)
	end,
	EnemyTargetNeeded = true,
}

a.spells["arcane_shot,if=focus>=67"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and x.Focus >= 67
	end,
	EnemyTargetNeeded = true,
}

a.spells["steady_shot,if=buff.pre_steady_focus.up&buff.steady_focus.remains<=5"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.PreSteadyFocus() and s.MyBuffDuration(53224--[[Steady Focus]], "player") <= 5 + x.GCD
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["steady_shot,if=buff.pre_steady_focus.up&buff.steady_focus.remains<6"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.PreSteadyFocus() and s.MyBuffDuration(53224--[[Steady Focus]], "player") < 6 + x.GCD
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["steady_shot,if=buff.steady_focus.remains<(action.steady_shot.cast_time+1)&!in_flight"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and s.BuffDuration(53224--[[Steady Focus]], "player") < (s.CastTime(53224--[[Steady Focus]]) + 1 + x.GCD) and not s.SpellDelay(56641--[[Steady Shot]], "target")
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["steady_shot"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["cobra_shot"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["cobra_shot,if=dot.serpent_sting.remains<6"] = {
	ID = 77767--[[Cobra Shot]],
	RequireMyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		return x.NoCC and s.MyDebuffDuration(1978--[[Serpent Sting]]) < 6 + x.GCD
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["rapid_fire,if=!buff.rapid_fire.up"] = {
	ID = 3045--[[Rapid Fire]],
	Buff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		if x.Boss then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and x.ActiveEnemy and not x.Stop
	end,
	RunLast = function(z)
		x.Stop = true
	end,
	NoGCD = true,
}

a.spells["aimed_shot,if=buff.master_marksman_fire.react"] = {

	ID = 82928--[[Aimed Shot!]],
	Override = function(z)
		return x.NoCC and s.Enemy("target") and s.BuffDuration(82926--[[Fire!]], "player") > x.GCD
	end,
	FlashID = {82928--[[Aimed Shot!]], 19434--[[Aimed Shot]],},
}

a.spells["aimed_shot"] = {
	ID = 19434--[[Aimed Shot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["aimed_shot,if=cast_time<1.7"] = {
	ID = 19434--[[Aimed Shot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and s.CastTime(19434--[[Aimed Shot]]) < 1.7
	end,
	EnemyTargetNeeded = true,
}

a.spells["Call Pet"] = {
	ID = 9--[[Call Pet]],
	Override = function(z)
		if not x.PetAlive and x.ShouldPermanentBuff then
			s.Flash({
				9--[[Call Pet]],
				883--[[Call Pet 1]],
				83242--[[Call Pet 2]],
				83243--[[Call Pet 3]],
				83244--[[Call Pet 4]],
				83245--[[Call Pet 5]],
				982--[[Revive Pet]]})
		end
	end,
}

a.spells["Mend Pet"] = {
	ID = 136--[[Mend Pet]],
	Buff = 136--[[Mend Pet]],
	BuffUnit = "pet",
	EarlyRefresh = 1,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.PetAlive and (s.HealthPercent("pet") < 80 or (s.HasGlyph(19573--[[Glyph of Mend Pet]]) and s.Debuff(nil, "pet", nil, nil, 1)))
	end,
	FlashColor = "Red",
}

a.spells["explosive_shot,if=buff.lock_and_load.react"] = {
	ID = 53301--[[Explosive Shot]],
	RequireBuff = 56453--[[Lock and Load]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["explosive_shot,if=cooldown_react"] = {
	ID = 53301--[[Explosive Shot]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["black_arrow,if=!ticking&target.time_to_die>=8"] = {
	ID = 3674--[[Black Arrow]],
	MyDebuff = 3674--[[Black Arrow]],
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 8
	end,
	EnemyTargetNeeded = true,
}

a.spells["Silencing Shot"] = {
	ID = 34490--[[Silencing Shot]],
	Debuff = 34490--[[Silencing Shot]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoGCD = true,
	FlashColor = "Aqua",
}

a.spells["virmens_bite_potion,if=buff.bloodlust.react|target.time_to_die<=60"] = {
	Type = "item",
	ID = 76089--[[Virmen's Bite]],
	Buff = 105697--[[Virmen's Bite]],
	NoGCD = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and x.Boss and (a.Bloodlust() or x.TimeToDie <= 60)
	end,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Concussive Shot"] = {
	ID = 5116--[[Concussive Shot]],
	CheckLast = function(z)
		return not x.Boss and s.Moving("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	EnemyTargetNeeded = true,
}

a.spells["Disengage"] = {
	ID = 781--[[Disengage]],
	CheckLast = function(z)
		return x.ActiveEnemy and s.MeleeDistance("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	NoGCD = true,
	NoRangeCheck = true,
	EnemyTargetNeeded = true,
}

a.spells["Feign Death"] = {
	ID = 5384--[[Feign Death]],
	NotWhileMoving = true,
	CheckLast = function(z)
		return (s.HealthPercent("player") < 50 and s.EnemyTargetingYou("target")) or x.ThreatPercent > 80
	end,
	FlashColor = "Red",
	NoGCD = true,
}

a.spells["Master's Call"] = {
	ID = 53271--[[Master's Call]],
	Check = function(z)
		return x.PetAlive
	end,
	CheckLast = function(z)
		local runSpeed
		_, runSpeed = GetUnitSpeed("player")
		runSpeed = runSpeed / 7 * 100
		if runSpeed < 100 then
			return true
		end
		local eventIndex = C_LossOfControl.GetNumEvents()
		while (eventIndex > 0) do
			local locType, _, _, _, _, timeRemaining = C_LossOfControl.GetEventInfo(eventIndex)
			if locType == "ROOT" then
				return timeRemaining > 1.5
			end
			eventIndex = eventIndex - 1
		end
	end,
	NoRangeCheck = true,
	FlashBlink = true,
	FlashColor = "Aqua",
	NoGCD = true,
}

a.spells["Widow Venom"] = {
	ID = 82654--[[Widow Venom]],
	Debuff = {
		82654--[[Widow Venom]],
		8680--[[Wound Poison]],
		115804--[[Mortal Wounds]],
		54680--[[Monstrous Bite]],
	},
	CheckLast = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Pink",
}

a.spells["Exhilaration"] = {
	ID = 109304--[[Exhilaration]],
	CheckLast = function(z)
		return a.HasTalent(109304--[[Exhilaration]]) and (s.HealthPercent("player") < 70 or (x.PetAlive and s.HealthPercent("pet") < 50))
	end,
	NoGCD = true,
	FlashColor = "Red",
}

a.spells["Counter Shot"] = {
	ID = 147362--[[Counter Shot]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoGCD = true,
	FlashColor = "Aqua",
}